#ifndef __CINT__
#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TFile.h"
#include "TH1.h"
#include "TCanvas.h"
#include "TView.h"
#include "TTUBE.h"
#include "TTUBS.h"
#include "TMath.h"
#include "TVector3.h"
#include "TClassMenuItem.h"
#include "GEMDisplay.h"
#include "JREvent.h"
#include "JCluster.h"
#include "kaltest/TKalDetCradle.h"
#include "TVKalDetector.h"
#include "TAttDrawable.h"
#include "EXVKalDetector.h"
#include "EXTPCKalDetector.h"
#include "JHitMaker.h"
#include "JHit.h"
#include "JTrack.h"
#include "JTrackMaker.h"
#include "JSimpleTrackFitter.h"
#include "JTripletTrackFinder.h"
#include "JHitCorrector.h"

#include <sstream>
#endif
#include "TNode.h"

//#define __MOVIE__
#define SEP_2010

////////////////////////////////
const Int_t kModuleToLookAt = 3;
////////////////////////////////

///////////////////////////////////////
//-----------------------------------//
//  Pulse shape (height/width) cuts  //
//-----------------------------------//
///////////////////////////////////////
#if 0
#if 1
//const int kMaxWidth  = 200;
const int kMaxWidth  = 60;
const int kMinWidth  =  7;
//const int kThreshold =  3; // 8;
const int kThreshold =  4;
//const int kThreshold =  5;
#else
const int kMaxWidth  = 15;
const int kMinWidth  =  5;
const int kThreshold =  90;
#endif
#endif
const double kCosThetaCut2 = 0.98;
const int    kMaxMissing2  =  15;
//const int    kMinNhits2    =  10;
const int    kMinNhits2    =  22;
const int    kZdiffCut2    = 5.0;
///////////////////////////////////////

#if 0
void PulseMonitor(Int_t /* event = 1 */, Int_t /* run = 68 */, Int_t /* part = 0*/, Bool_t reset = true) {
#else
void PulseMonitor(Int_t event = 1, Int_t run = 68, Int_t part = 0, Bool_t reset = true) {
#endif
  if (reset) {
    gROOT->LoadMacro("SetGlobalStyle.C");
    gROOT->ProcessLine("SetGlobalStyle()");
    gStyle->SetOptStat(1);
    gStyle->SetOptTitle(0);
    gStyle->SetPadBottomMargin(0.);
    gStyle->SetPadTopMargin(0.);
    gStyle->SetPadLeftMargin(0.);
    gStyle->SetPadRightMargin(0.);
    gStyle->SetPalette(1);
  }
#if 1
  Int_t kNRow = 5;
  Int_t kNCol = 6;
#endif

  GEMDisplay *gemdisplay = GEMDisplay::GetInstance();
  gemdisplay->GetCurFile()->cd();
  gemdisplay->GetCanvas()->Clear();
  gemdisplay->GetCanvas()->Divide(kNCol, kNRow, 0, 0);

  JREvent *ep = gemdisplay->GetCurEvent();
  JCluster::SetSortBy(JCluster::kDet);
  ep->Sort(); // sort clusters from upstream to downstream

  //JHitCorrector::GetInstancePtr("../../trackmaker/Results/deltax2.dat");
  JHitCorrector::GetInstancePtr("../../trackmaker/Results/deltax0.dat");
  JHitMaker        hitmaker;
  TKalDetCradle    lp1;
  EXTPCKalDetector tpcdet;
  lp1.Install(tpcdet);
  //lp1.SwitchOffMS();

  JSimpleTrackFitter fitter(&tpcdet);
  JTrack::SetTrackFitter(&fitter);

  JTrackMaker trackmaker;
  trackmaker.SetTrackFinder(new JTripletTrackFinder(&trackmaker));
  JTripletTrackFinder::SetMinNhits   (kMinNhits2);
  JTripletTrackFinder::SetMaxMissing (kMaxMissing2);
  JTripletTrackFinder::SetCosThetaCut(kCosThetaCut2);
  JTripletTrackFinder::SetZdiffCut(kZdiffCut2);

  //trackmaker.KillModule(1);
  //trackmaker.KillModule(3);
  //trackmaker.KillModule(6);

  trackmaker.SetModuleToLookAt(-1);
  trackmaker.SetLayerToLookAt(-1);

  hitmaker.MakeHits(ep);
  //hitmaker.DebugPrint("detail");
  trackmaker.MakeTracks(&hitmaker);
#if 1
  //trackmaker.DebugPrint("detailed");
#endif
  //--
  // Loop over tracks
  //--
  const Int_t kTimeOff = 10;
  TIter nexttrack(&trackmaker); 
  JTrack *tp;
  while ((tp = static_cast<JTrack *>(nexttrack()))) {
    cerr << "track tp=" << (void *)tp << endl;
    tp->DebugPrint();
    TIter nexthit(tp);
    JHit *hp;
    while ((hp = static_cast<JHit *>(nexthit()))) {
      TIter nextpulse(hp);
      Double_t qmax  = -9999.;
      JCluster *cppk = 0;
      JCluster *cp; 
      while ((cp = static_cast<JCluster *>(nextpulse()))) {
        if (cp->GetCharge() > qmax) {
          cppk = cp;
	  qmax = cp->GetCharge();
	}
      }
      cp = cppk;
      // Draw the peak pulse
      Int_t module = cp->GetModuleID();
      Int_t layer  = cp->GetLayerID();
      if (module != kModuleToLookAt ) continue;
      //--
      // Fill the histogram
      //--
      stringstream hname;
      hname.str("");
      hname.clear();
      hname << "hPulse" << module << "_" << layer << ends;
      TH1F *pp = static_cast<TH1F *>(gROOT->FindObject(hname.str().data()));
      if (!pp) {
        pp = new TH1F(hname.str().data(),hname.str().data(),50,0.,50.);
      } else {
        pp->Reset();
      }
      pp->SetMinimum(-10.);
      pp->SetMaximum(1090.);
      pp->SetBit(TObject::kCanDelete);
      pp->GetXaxis()->SetTitle("Time bin");
      pp->GetYaxis()->SetTitle("ADC count");
      Int_t time = cp->GetStartTime();
      for (Int_t i=0; i<cp->GetLength(); i++) {
        //pp->SetBinContent(i+time, cp->operator[](i));
        //pp->SetBinContent(i+kTimeOff, cp->operator[](i));
        pp->SetBinContent(i+cp->GetStartTime()-cp->GetTime()+kTimeOff, cp->operator[](i));
      }
      gemdisplay->GetCanvas()->cd(layer+1);
      pp->Draw();
    }
  }

  gemdisplay->GetCanvas()->Modified();
  gemdisplay->GetCanvas()->Update();
}
